<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Model\Util\NovoCodigo;
use Teknisa\CRM\Service\RelatParcNegocioService;

class RelatParcNegocioController extends TeknisaCrud {
    protected $parameterBagColumns = array('P_NRORG', 'P_CRM_NRORGTRAB', 'NRREVENDA');
    protected $environment;
    
    protected $dataSourceGet = 'getRelatParcNegocio';
    
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, RelatParcNegocioService $relatParcNegocioService, $environment){
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->relatParcNegocioService = $relatParcNegocioService;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public function getRelatParcNegocio(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $conditions = $filterData->getConditions();
            $filterData = new FilterCriteria($this->dataSourceGet, $conditions);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

}