function MapController(restEngine, ScreenService, ZHPromise, templateManager) {
    
    var map;
    var markers;
    var dataForReplace = [];

    this.getSRC = function(){
        var widget = templateManager.container.getActiveWidget();
        return widget.mapUrl;
    };
    
    this.getMap = function() {
        return "https://maps.googleapis.com/maps/api/js?key=AIzaSyA5vRtFek_17V-Q6t4bhTYpSCSunxDPzzM&callback=initMap";
    };
    
    this.initMap = function (filteredData, filter) {
        var center = {
            lat: -14.235004,
            lng: -51.925280
        };
        templateManager.mapPreviousPosition = [];
    
        ScreenService.changeLoadingMessage('Carregando Mapa');
        ScreenService.showLoader();
    
        map = new google.maps.Map(document.getElementById('map'), {
            center: center,
            zoom: 4
        });
    
        if (Util.isEmptyOrBlank(filter)) {
            filter = [
                {
                    "name": "CRM_NRORGTRAB",
                    "operator": "=",
                    "value": getLocalVar('CRM_NRORGTRABALHO')
                }
            ];
        }
    
        this.buildLocations(filteredData, filter).then(function (locations) {
            var markersInMap = [];
            this.clearCurrentList();
            templateManager.container.getWidget('list').dataSource.data = locations;
            templateManager.container.getWidget('list').dataSource.reload();
            this.createParents();
            markers = locations.map(function (location, i) {
                var icon = this.buildIcon(location['CDTIPOPARCPRINCIPAL'], location['NRTERMOPORTUNI']);
                var position = new google.maps.LatLng(parseFloat(location['DSLATITUDE']), parseFloat(location['DSLONGITUDE']));
                if (this.isLocationFree(position, markersInMap)) {
                    markersInMap.push(position);
                    return new google.maps.Marker({
                        position: position,
                        map: map,
                        icon: icon,
                        information: location
                    });
                } else {
                    return 'duplicated';
                }
            }.bind(this)).filter(function (marker) { return marker !== 'duplicated' });
            templateManager.mapMarkers = markers;
            templateManager.map = map;
    
            var markerCluster = new MarkerClusterer(map, markers, {
                imagePath: 'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m',
                gridSize: 45
            });
            google.maps.event.addListener(markerCluster, "clusterclick", function (cluster) {
                if (!templateManager.mapPreviousPosition.find(function (n) { return n.zoom === templateManager.map.getZoom() }))
                    templateManager.mapPreviousPosition.push({ "center": templateManager.map.getCenter(), "zoom": templateManager.map.getZoom() });
            });
    
            google.maps.event.addListener(map, "dblclick", function () {
                if (!templateManager.mapPreviousPosition.find(function (n) { return n.zoom === templateManager.map.getZoom() }))
                    templateManager.mapPreviousPosition.push({ "center": templateManager.map.getCenter(), "zoom": templateManager.map.getZoom() });
            });
    
            templateManager.mapClusters = markerCluster;
    
            markers.forEach(function (marker) {
                var nome = (Util.isEmptyOrBlank(marker['information']['NMPRINCIPALPARC'])) ? '' : marker['information']['NMPRINCIPALPARC'];
                var endereco = (Util.isEmptyOrBlank(marker['information']['DSENDERECO'])) ? '' : marker['information']['DSENDERECO'];
                var interlocutor = (Util.isEmptyOrBlank(marker['information']['NMINTERLOCUTOR'])) ? '' : marker['information']['NMINTERLOCUTOR'];
                var telefone = (Util.isEmptyOrBlank(marker['information']['DSTELEFONE'])) ? '' : marker['information']['DSTELEFONE'];
                if (telefone)
                    telefone = '(' + telefone.slice(0, 2) + ')' + ' ' + telefone.slice(2, 6) + '-' + telefone.slice(6, telefone.length);
    
                var contentInfoWindow =
                    '<style> @import url("https://fonts.googleapis.com/css?family=Roboto:300,900");' +
                    'h1,h2,h3,h4{font-family:"Roboto", sans-serif; color:#414141}' +
                    'h1{font-size:11px;font-weight:900; line-height:0.9}' +
                    'h2{font-size:9.5px;font-weight:300; color:#232323; line-height:1.2}' +
                    'h3{font-size:10px;font-weight:300; line-height:1.2}' +
                    'h4{font-size:9.5px;font-weight:300; line-height:1.2}' +
                    '</style>' +
                    '<div id="infoWindowMap" style="width:150px; height:auto; border-radius:10%;">' +
                    '<h1>' + nome + '</h1>' +
                    '<h2>' + endereco + '</h2>' +
                    '<h3>' + interlocutor + '</h2>' +
                    '<h4>' + telefone + '</h3>' +
                    '</div>';
    
                var infoWindow = new google.maps.InfoWindow({
                    content: contentInfoWindow
                });
    
                marker.addListener('mouseover', function () {
                    infoWindow.open(map, marker);
                });
                marker.addListener('mouseout', function () {
                    infoWindow.close(map, marker);
                });
                marker.addListener('click', function () {
                    var widget = templateManager.container.getWidget('iframe');
                    widget.currentRow.NRPARCNEGOCIO = marker['information']['NRPARCNEGOCIO'];
                    ScreenService.openPopup(templateManager.container.getWidget('INFOPARCEIRO'));
                });
            }.bind(this));
    
            if (filteredData)
                this.closeFilter();
    
            ScreenService.hideLoader();
            ScreenService.restoreDefaultLoadingMessage();
        }.bind(this));
    };
    
    this.buildLocations = function (filteredData, filter) {
        var promise = ZHPromise.defer();
        var widMapa = templateManager.container.getWidget('mapa');
    
        if (typeof filter === 'undefined' || filter === null) {
            filter = null;
        }
    
        this.getLocations(filter).then(function (dataSet) {
            var locations = dataSet.dataset.locations;
            widMapa.dataSource.data = dataSet.dataset.locations;
            promise.resolve(locations.filter(function (n) {
                return (n['DSLATITUDE'] !== null && n['DSLONGITUDE'] !== null);
            }));
        }.bind(this));
    
        return promise.promise;
    };
    
    this.getLocations = function (filter) {
        return restEngine.post('/getLocationsForMap', { 'requestType': 'FilterData', 'filter': filter }, null, null, null, null, 900000000);
    };
    
    this.buildList = function(markers) {
        var list = templateManager.container.getWidget('list');
        var data = list.dataSource.data;
        var maxID = 100;
        
        markers.forEach(function(parceiro){
            parceiro = parceiro['information'];
            var parent = parceiro['CDTIPOPARCPRINCIPAL'] === 'CLIENTE' ? "30" : parceiro['CDTIPOPARCPRINCIPAL'] === 'PROSPECT' ? "20" : "10";
            
            var itemForList = {
                "ID":(maxID+1).toString(),
                "NMPRINCIPALPARC":  parceiro['NMPRINCIPALPARC'],
                "DSENDERECO":       parceiro['DSENDERECO'],
                "NMINTERLOCUTOR":   parceiro['NMINTERLOCUTOR'],
                "DSTELEFONE":       parceiro['DSTELEFONE'],
                "PARENT":           parent,
                "DSLATITUDE":       parceiro['DSLATITUDE'],
                "DSLONGITUDE":      parceiro['DSLONGITUDE'],
                "NRPARCNEGOCIO":    parceiro['NRPARCNEGOCIO']
            };
            data.push(itemForList);    
            maxID++;
        }.bind(this));
        
        
        list.dataSource.data = data;
        list.reload();
    };
    
    this.clearCurrentList = function() {
        var list = templateManager.container.getWidget('list');
        var data = list.dataSource.data;
        data = data.splice(0, 1);
        list.dataSource.data = data;
        list.reload();
    };
    
    this.createParents = function (exists) {
        exists = exists || false;
        var widget = templateManager.container.getWidget('list');
        var data = widget.dataSource.data;
        var title = lead = prospect = cliente = null;
    
        data.forEach(function (parceiro, parcIndex) {
            if (parceiro.CDTIPOPARCPRINCIPAL == 'CLIENTE') {
                parceiro.PARENT = '30';
            } else if (parceiro.CDTIPOPARCPRINCIPAL == 'PROSPECT') {
                parceiro.PARENT = '20';
            } else if (parceiro.CDTIPOPARCPRINCIPAL == 'CANDIDATO') {
                parceiro.PARENT = '10';
            }
    
            if (parceiro.DSLATITUDE == null || parceiro.DSLONGITUDE == null) {
                dataForReplace.push(parceiro);
                delete data[parcIndex];
            }
        });
    
        if (exists == false) {
            title = {
                "ID": "1",
                "NMPRINCIPALPARC": "Parceiros Listados",
                "PARENT": null,
                "preventClick": true
            };
            lead = {
                "ID": "10",
                "NMPRINCIPALPARC": "Lead",
                "PARENT": null,
                "opened": false,
                "preventClick": true
            };
            prospect = {
                "ID": "20",
                "NMPRINCIPALPARC": "Prospect",
                "PARENT": null,
                "opened": false,
                "preventClick": true
            };
            cliente = {
                "ID": "30",
                "NMPRINCIPALPARC": "Cliente",
                "PARENT": null,
                "opened": false,
                "preventClick": true
            };
        }
        data.push(title, lead, prospect, cliente);
    };
    
    this.openFilter = function() {
        var filterMap = templateManager.container.getWidget('filterMap');
        if(Util.isEmptyOrBlank(filterMap.currentRow.IDATIVO)) {
            filterMap.currentRow.IDATIVO = 'S';
        }
        if(Util.isEmptyOrBlank(filterMap.currentRow.CDOPERADOR)) {
            filterMap.currentRow.CDOPERADOR = templateManager.LOGGEDOPERATOR.CDOPERADOR;
            filterMap.currentRow.NMOPERADOR = templateManager.LOGGEDOPERATOR.NMOPERADOR;
        }
        ScreenService.openPopup(filterMap);
    };
    
    this.closeFilter = function() {
        var filterMap = templateManager.container.getWidget('filterMap');
        filterMap.isVisible = false;
        ScreenService.closePopup(true);
        templateManager.container.getWidget('mapa').activate();
    };
    
    this.colorLuminance = function(hex, lum) {
        
    	// validate hex string
    	hex = String(hex).replace(/[^0-9a-f]/gi, '');
    	if (hex.length < 6) {
    		hex = hex[0]+hex[0]+hex[1]+hex[1]+hex[2]+hex[2];
    	}
    	lum = lum || 0;
    
    	// convert to decimal and change luminosity
    	var rgb = "#", c, i;
    	for (i = 0; i < 3; i++) {
    		c = parseInt(hex.substr(i*2,2), 16);
    		c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
    		rgb += ("00"+c).substr(c.length);
    	}

    	return rgb;
    };
    
    this.buildIcon = function(tipoParc, termParc) {
        var templateForProspect = [
                '<?xml version="1.0" encoding="utf-8"?> <!-- Generator: Adobe Illustrator 16.0.0, SVG Export Plug-In . SVG Version: 6.00 Build 0) --> <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="512px" height="512px" viewBox="0 0 512 512" enable-background="new 0 0 512 512" xml:space="preserve"> <g> <path fill="{{fillMarker}}" stroke="{{strokeMarker}}" stroke-miterlimit="10" d="M256,0C167.641,0,96,71.625,96,160 c0,24.75,5.625,48.219,15.672,69.125C112.234,230.313,256,512,256,512l142.594-279.375C409.719,210.844,416,186.156,416,160 C416,71.625,344.375,0,256,0z M160,160 M352,160"/> </g> <g> <path fill="{{fillLetter}}" d="M242.084,222.002V286h-29.168V109.513h53.537c20.266,0,35.979,4.719,47.137,14.153 c11.158,9.437,16.738,22.729,16.738,39.876c0,17.149-5.826,31.18-17.477,42.091c-11.65,10.913-27.404,16.369-47.26,16.369H242.084z M242.084,133.635v64.491h18.953c12.555,0,22.111-2.912,28.678-8.738c6.563-5.825,9.846-14.031,9.846-24.615 c0-20.757-12.061-31.138-36.185-31.138H242.084z"/> </g> </svg>'
                ].join('\n');;
        var templateForLead     = [
                '<?xml version="1.0" encoding="utf-8"?> <!-- Generator: Adobe Illustrator 16.0.0, SVG Export Plug-In . SVG Version: 6.00 Build 0) --> <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="512px" height="512px" viewBox="0 0 512 512" enable-background="new 0 0 512 512" xml:space="preserve"> <g> <path fill="{{fillMarker}}" stroke="{{strokeMarker}}" stroke-miterlimit="10" d="M256,0C167.641,0,96,71.625,96,160 c0,24.75,5.625,48.219,15.672,69.125C112.234,230.313,256,512,256,512l142.594-279.375C409.719,210.844,416,186.156,416,160 C416,71.625,344.375,0,256,0z M160,160 M352,160"/> </g> <g> <path fill="{{fillLetter}}" d="M310.637,286h-97.721V109.513h29.292v151.75h68.429V286z"/> </g> </svg>'
                ].join('\n');;
        var templateForClient   = [
                '<?xml version="1.0" encoding="utf-8"?> <!-- Generator: Adobe Illustrator 16.0.0, SVG Export Plug-In . SVG Version: 6.00 Build 0) --> <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="512px" height="512px" viewBox="0 0 512 512" enable-background="new 0 0 512 512" xml:space="preserve"> <g> <path fill="{{fillMarker}}" stroke="{{strokeMarker}}" stroke-miterlimit="10" d="M256,0C167.641,0,96,71.625,96,160 c0,24.75,5.625,48.219,15.672,69.125C112.234,230.313,256,512,256,512l142.594-279.375C409.719,210.844,416,186.156,416,160 C416,71.625,344.375,0,256,0z M160,160 M352,160"/> </g> <g> <path fill="{{fillLetter}}" d="M310.492,267.616c-13.291,6.892-29.826,10.338-49.6,10.338c-25.599,0-46.112-8.081-61.537-24.246 c-15.426-16.163-23.138-37.372-23.138-63.629c0-28.225,8.677-51.033,26.03-68.429c17.354-17.394,39.28-26.092,65.783-26.092 c17.066,0,31.219,2.421,42.461,7.262v29.045c-11.898-7.056-25.025-10.584-39.383-10.584c-19.12,0-34.606,6.113-46.462,18.338 c-11.857,12.227-17.784,28.553-17.784,48.983c0,19.446,5.538,34.934,16.615,46.46c11.077,11.529,25.64,17.292,43.691,17.292 c16.656,0,31.096-3.938,43.322-11.815V267.616z"/> </g> <g> </g> </svg>'
                ].join('\n');;
        var color, template;
        
        if(tipoParc === 'CLIENTE'){
            color = '#48db81';
            template = templateForClient;
        }else if(tipoParc === 'CANDIDATO'){
            color = '#ffc64a';
            template = templateForLead;
        }else{
            color = '#1893db';
            template = templateForProspect;
        }
        
        if(!Util.isEmptyOrBlank(templateManager.termometroFilter)){
            templateManager.termometroFilter.forEach(function(termometro){
                if(termometro.nrtermoportuni === termParc)
                    color = termometro.cor;
            });
        }
        
        var svg = this.replaceTemplateWithColor(template, color);        
        return { url: 'data:image/svg+xml;charset=UTF-8,' + encodeURIComponent(svg), scaledSize: new google.maps.Size(35, 35), color:color }   
    };
    
    this.chooseColor = function(type) {
        templateManager.mapMarkers.forEach(function(marker, index, markers){
            if(type === 'oportunidade'){
                var icon = marker.getIcon();    
                icon.url = this.replaceTemplateWithColor(icon.url, '#42f480', icon.color);
                marker.setIcon(icon);
            }
        }.bind(this));
    };
    
    this.replaceTemplateWithColor = function(template, color, actualColor) {
        var replacer;
        if(actualColor){
            replacer = actualColor.replace('#', '');
            color = color.replace('#', '');
        }
        var svg = template.replace(Util.isEmptyOrBlank(replacer) ? '{{fillMarker}}' : replacer, color);
        svg.replace(Util.isEmptyOrBlank(replacer) ? '{{strokeMarker}}'  : replacer, this.colorLuminance(color, -0.2));
        svg.replace(Util.isEmptyOrBlank(replacer) ? '{{fillLetter}}'  : replacer, this.colorLuminance(color, -0.2));
        return svg;
    };
    
    this.centerMarker = function(args) {
        var currentMarker = args.owner.currentRow;
        if(!currentMarker.preventClick){
            var lat = currentMarker.DSLATITUDE;
            var lng = currentMarker.DSLONGITUDE;
            if(!templateManager.mapPreviousPosition.find(function(n){return n.zoom === templateManager.map.getZoom()}))
                templateManager.mapPreviousPosition.push({"center":templateManager.map.getCenter(), "zoom":templateManager.map.getZoom()});
            
            var center = new google.maps.LatLng(parseFloat(lat), parseFloat(lng));
            
            var bounds = new google.maps.LatLngBounds();
            bounds.extend(center);
            templateManager.map.panTo(center);
            templateManager.map.fitBounds(bounds);
            templateManager.map.setZoom(15);
            
            templateManager.mapMarkers.forEach(function(marker){
                google.maps.event.trigger(marker, 'mouseout');
                if(marker['information']['NRPARCNEGOCIO'] === currentMarker['NRPARCNEGOCIO'])
                    google.maps.event.trigger(marker, 'mouseover');
            });
        }
    };
    
    this.smoothZoom = function(map, max, cnt) {
        if(cnt == max){
            return
        }else if (cnt > max) {
             var z = google.maps.event.addListener(map, 'zoom_changed', function(event){
                google.maps.event.removeListener(z);
                this.smoothZoom(map, max, cnt - 1);
            }.bind(this));
            
            setTimeout(function(){map.setZoom(cnt)}, 40); 
        }
    }; 
    
    this.geocodePartners = function(widget) {
        this.getLocations().then(function(dataSet){
            var locations = dataSet.dataset.locations;
            var geocoder = new google.maps.Geocoder();
            var geocoded = [];
            var partnersWithoutLat = (locations.filter(function(n){ return n['DSLONGITUDE'] == null }));
            partnersWithoutLat.forEach(function(partner, index){
                geocoder.geocode({'address': (partner['ENDERECO']).toString()}, function(results, status) {
                    if (status === 'OK') {
                        geocoded.push({parcnegocio: partner['NRPARCNEGOCIO'], nrorg: partner['NRPARCNEGOCIO'], lat: results[0].geometry.location.lat(), lng:results[0].geometry.location.lng()});
                    }
                    if(index === partnersWithoutLat.length-1)
                        restEngine.post('/geocode', {
                                requestType: "DataSet",
                                dataset: geocoded
                        }).
                        then(function(response){
                            
                        }); 
                }.bind(this)); 
            }.bind(this));
        });
    };
    
    this.readOnlyFieldValorOport = function(widget) {
        var fieldValorOportDe = widget.getField('VROPORT');
        var fieldValorOportAte = widget.getField('VROPORTATE');
        
        var valorDe;
        var valorAte;
        
        if(fieldValorOportDe.value()) {
            valorDe = fieldValorOportDe.value().replace(/[^0-9]/g, '').slice(0, -2);
        }
        
        if(fieldValorOportAte.value()) {
            valorAte = fieldValorOportAte.value().replace(/[^0-9]/g, '').slice(0, -2);
        }
        
        if(Util.isEmptyOrBlank(fieldValorOportDe.value())){
            // fieldValorOportAte.readOnly = true;
            // fieldValorOportAte.clearValue();
            fieldValorOportAte.validations.minCurrency.limit = '';
        }else{
            // fieldValorOportAte.readOnly = false;
            // fieldValorOportAte.clearValue();
            fieldValorOportAte.validations.minCurrency.limit = valorDe;
        }
    };
    
    this.buildFilterValorOport = function() {
        var widget = templateManager.container.getActiveWidget();
        var fieldValorOportDe = widget.getField("VROPORT");
        var fieldValorOportAte = widget.getField("VROPORTATE");
        
        var valorDe, valorAte;
        
        if(Util.isEmptyOrBlank(fieldValorOportDe.value())){
            fieldValorOportDe.value("R$0,00");
        }
        
        if(Util.isEmptyOrBlank(fieldValorOportAte.value())){
            fieldValorOportAte.value("R$10000000,00");
        }
        
        if(fieldValorOportDe.value()) {
            valorDe = fieldValorOportDe.value().toString().replace(/[^0-9]/g, '').slice(0, -2);
        }
        if(fieldValorOportAte.value()) {
            valorAte = fieldValorOportAte.value().toString().replace(/[^0-9]/g, '').slice(0, -2);
        }
        var filters = [];
        
        if(!Util.isEmptyOrBlank(valorDe) && !Util.isEmptyOrBlank(valorAte)){
            var frame = templateManager.container.getWidget('iframe');
            filters.push(widget.currentRow);
            
            frame.dataSourceFilter = filters;
            var filterBetween = {
                "name":"VROPORT",
                "operator":"BETWEEN",
                "value": [valorDe, valorAte]
            };
            
            frame.dataSourceFilter[0].push(filterBetween);
            delete frame.dataSourceFilter[0].$error;
        }else{
            widget.dataSourceFilter.forEach(function(filter, index){
                if(filter.name !== "VROPORT" && filter.name !== "VROPORTATE")
                    filters.push(filter);
            });
            widget.dataSourceFilter = filters;    
        }
        
        this.unMaskPhone(widget);
        // this.checkTermometro(widget);
    };
    
    this.unMaskPhone = function(widget) {
        widget.dataSourceFilter.forEach(function(filter){
            if(filter.name === "TELEFONE"){
                filter.value = filter.value.replace(/[^0-9]/g, '');
                filter.value = '%' + filter.value + '%';
            }
        });
    };

    this.goBackZoom = function() {
        var center = {
            lat: -14.235004,
            lng: -51.925280    
        };
        
        var previousCenter = center;
        var previousZoom   = 4;
    
        this.smoothZoom(templateManager.map, previousZoom-1, templateManager.map.getZoom());
        templateManager.map.panTo(previousCenter);
        
        templateManager.mapMarkers.forEach(function(marker){
                google.maps.event.trigger(marker, 'mouseout');
        });
    };
    
    this.checkTermometro = function(widget) {
        var filters = widget.dataSourceFilter;
        filters.forEach(function(filter){
            if(filter.name === "NRTERMOPORTUNI"){
                templateManager.termometroFilter = [];
                filter.value.forEach(function(termometro, index){
                    templateManager.termometroFilter.push({"nrtermoportuni": termometro, "cor":widget.dataSourceFilter.filter(function(f){return f.name === "DSCORTERMOMETRO"})[0].value[index]});
                });                                                                 
            }    
        });
        
        
    };
    
    this.isLocationFree = function(position, markersInMap) {
        var lat = position.lat();
        var lng = position.lng();
        
        markersInMap.forEach(function(markerInMap){
            if(markerInMap.position.lat() === lat && markerInMap.position.lng() === lng)
                return false;
        });
        
        return true;
    };
    
    this.atualizaLatLong = function(dataAux, registros) {
        var widget = templateManager.container.getWidget('mapa');
       
        restEngine.post('/atualizaLatLong', {requestType: "DataSet", dataset: dataAux}).then(function(response) {
            if (!Util.isEmptyOrBlank(dataAux) && widget.geocodeError) {
                ScreenService.notificationMessage('Foram atualizados somente ' + registros + ' porque o limite de pesquisa por latitude e longitude foi excedido.', "alert");
            }
            this.aplicarFiltro();
            this.createParents(true);
            templateManager.container.getWidget('list').dataSource.reload();
        }.bind(this));
    };
    
    
    this.setLatLong = function () {
        ScreenService.confirmMessage("Todos os parceiros relacionados ao operador filtrado que não possuam latitude e longitude terão seus respectivos dados atualizados conforme o endereço principal atual, deseja continuar? (Este processo pode demorar algum tempo)", 'CONFIRMATION',
            function (success) {
                var widget = templateManager.container.getWidget('mapa');
                var data = widget.dataSource.data;
                var geocoder = new google.maps.Geocoder();
                var dataAux = [];
    
                data.forEach(function (parceiro) {
                    if (!parceiro.DSLONGITUDE && !parceiro.DSLATITUDE) {
                        dataAux.push(parceiro);
                    }
                });
    
                // Repete o processo até que não existam mais parceiros no dataSource ou até que o geocoder tenha dado erro
                var contador = 0; var registros = 0;
                ScreenService.showLoader();
                var interval = setInterval(function () {
                    if (contador + 1 >= dataAux.length && contador !== 0) {
                        clearInterval(interval);
                        ScreenService.hideLoader();
                        if (!Util.isEmptyOrBlank(dataAux)) {
                            ScreenService.confirmMessage("Foram encontrados " + dataAux.length + " parceiros, deseja continuar?", "CONFIRMATION", function (success) {
                                this.atualizaLatLong(dataAux, registros);
                            }.bind(this));
                        } else if (Util.isEmptyOrBlank(dataAux) && widget.geocodeError) {
                            ScreenService.showMessage(widget.geocodeError, "ERROR");
                        }
                    } else {
                        if (Util.isEmptyOrBlank(widget.geocodeError)) {
                            geocoder.geocode({ 'address': dataAux[contador].DSENDERECO }, function (results, status) {
                                if (dataAux[contador]) {
                                    if (status == google.maps.GeocoderStatus.OK) {
                                        var latitude = results[0].geometry.location.lat();
                                        var longitude = results[0].geometry.location.lng();
                                        dataAux[contador].DSLATITUDE = latitude;
                                        dataAux[contador].DSLONGITUDE = longitude;
                                        registros++;
                                    } else if (status == 'OVER_QUERY_LIMIT') {
                                        widget.geocodeError = 'Limite de pesquisa por latitude e longitude por tempo ou diário excedido! Espere alguns minutos e tente novamente ou tente novamente em outro dia.';
                                    }
                                }
                            });
                        }
                        contador++;
                    }
                }.bind(this), 1500);
    
            }.bind(this));
    
    };
    
    this.closeModal = function() {
        var map = templateManager.container.getWidget('mapa');
        ScreenService.closePopup(true);
        map.activate();
    };
    
    this.aplicarFiltro = function() {
        // foi feito um filtro a mão pois o filtro do zeedhi só pega para uma widget com dataSource, e neste caso, o dataSource é chamado em um função
        var widget = templateManager.container.getWidget('filterMap');
        var currentRow = widget.currentRow;
        var filtro = [];
        
        if(!Util.isEmptyOrBlank(currentRow.CDTIPOPARCPRINCIPAL)) {
            filtro.push(
                {
                    "name": "CDTIPOPARCPRINCIPAL",
                    "operator": "=",
                    "value": currentRow.CDTIPOPARCPRINCIPAL
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.CDTIPOINSCRICAO)) {
            filtro.push(
                {
                    "name": "CDTIPOINSCRICAO",
                    "operator": "=",
                    "value": currentRow.CDTIPOINSCRICAO
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.NRINSCRICAOPARC)) {
            filtro.push(
                {
                    "name": "NRINSCRICAOPARC",
                    "operator": "LIKE_I",
                    "value": currentRow.NRINSCRICAOPARC
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.NMPRINCIPALPARC)) {
            filtro.push(
                {
                    "name": "NMPRINCIPALPARC",
                    "operator": "LIKE_I",
                    "value": currentRow.NMPRINCIPALPARC
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.NMSECUNDARIPARC)) {
            filtro.push(
                {
                    "name": "NMSECUNDARIPARC",
                    "operator": "LIKE_I",
                    "value": currentRow.NMSECUNDARIPARC
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.CDPAIS)) {
            filtro.push(
                {
                    "name": "CDPAIS",
                    "operator": "=",
                    "value": currentRow.CDPAIS
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.SGESTADO)) {
            filtro.push(
                {
                    "name": "SGESTADO",
                    "operator": "=",
                    "value": currentRow.SGESTADO
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.CDMUNICIPIO)) {
            filtro.push(
                {
                    "name": "CDMUNICIPIO",
                    "operator": "=",
                    "value": currentRow.CDMUNICIPIO
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.CDBAIRRO)) {
            filtro.push(
                {
                    "name": "CDBAIRRO",
                    "operator": "=",
                    "value": currentRow.CDBAIRRO
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.IDATIVO)) {
            filtro.push(
                {
                    "name": "IDATIVO",
                    "operator": "=",
                    "value": currentRow.IDATIVO
                }
            );
        } if(!Util.isEmptyOrBlank(currentRow.CDOPERADOR)) {
            filtro.push(
                {
                    "name": "CDOPERADOR",
                    "operator": "=",
                    "value": currentRow.CDOPERADOR
                }
            );
        }
        
        filtro.push(
            {
                "name": "CRM_NRORGTRAB",
                "operator": "=",
                "value": getLocalVar('CRM_NRORGTRABALHO')
            }
        );
        
        this.initMap('default', filtro);
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('MapController', MapController);
});